//
//  KeyboardTestViewController.swift
//  Stocks
//
//  Created by Joe Fabisevich on 12/28/18.
//  Copyright © 2018 scenee. All rights reserved.
//

import UIKit

final class StackViewContainingViewController: UIViewController {

    private let stackView: UIStackView = {
        let stackView = UIStackView()
        stackView.distribution = .fill
        stackView.alignment = .fill
        stackView.axis = .vertical
        stackView.spacing = 0.0

        return stackView
    }()

    private let topView: UIView = {
        let view = UIView()
        view.backgroundColor = UIColor.blue

        return view
    }()

    private let textField: UITextField = {
        let textField = UITextField()
        textField.backgroundColor = UIColor.white
        textField.textAlignment = .center
        textField.placeholder = "Placeholder"
        textField.autocapitalizationType = .words
        textField.returnKeyType = .done

        return textField
    }()

    override func viewDidLoad() {
        super.viewDidLoad()

        self.setup()
    }

    func setup() {
        self.stackView.translatesAutoresizingMaskIntoConstraints = false
        self.view.addSubview(self.stackView)

        self.stackView.addArrangedSubview(self.topView)

        let flexibleSpacerView = UIView()
        flexibleSpacerView.heightAnchor.constraint(greaterThanOrEqualToConstant: 1.0).isActive = true
        self.stackView.addArrangedSubview(flexibleSpacerView)

        self.stackView.addArrangedSubview(self.textField)

        self.setupConstraints()

        self.textField.becomeFirstResponder()
    }

    func setupConstraints() {
        if #available(iOS 11.0, *) {
            let keyboardLayoutGuide = self.view.addKeyboardLayoutGuide()
            //            self.stackView.pinToSuperview([.leading, .trailing, .top])
            //            self.stackView.pin(edge: .bottom, toEdge: .top, of: keyboardLayoutGuide, priority: .required)

            let stackViewConstraints: [NSLayoutConstraint] = [
                self.stackView.leadingAnchor.constraint(equalTo: self.view.safeAreaLayoutGuide.leadingAnchor),
                self.stackView.trailingAnchor.constraint(equalTo: self.view.safeAreaLayoutGuide.trailingAnchor),
                self.stackView.topAnchor.constraint(equalTo: self.view.safeAreaLayoutGuide.topAnchor),
                self.stackView.bottomAnchor.constraint(equalTo: keyboardLayoutGuide.topAnchor),
                //                self.stackView.bottomAnchor.constraint(equalTo: self.view.safeAreaLayoutGuide.bottomAnchor),
            ]

            self.topView.heightAnchor.constraint(equalToConstant: 200.0).isActive = true
            self.textField.heightAnchor.constraint(equalToConstant: 44.0).isActive = true

            NSLayoutConstraint.activate(stackViewConstraints)
        } else {
            // Go away, we're just testing this code.
        }
    }
}
